/* * * * * * * * * * * * * *  INITIALIZATIONS  * * * * * * * * * * * * * *\
 * findgfx () [char/boolean]                                             *
 * gfxmode () [void]                                                     *
 * [global var] gpage [unsigned int]                                     *
 *   address of the current active page                                  *
 *                                                                       *
 * * * * * * * * * * * * * * * *  DRAWING  * * * * * * * * * * * * * * * *
 * pset (x,y,color) [void macro asm]                                     *
 * point (x,y) [unsigned char asm]                                       *
 * cls (color) [void asm redefined]                                      *
 * pset4 (x,y,color) [void macro asm]                                    *
 *   sets four pixels at the same time, which is faster; x % 4 must be 0 *
 *                                                                       *
 * * * * * * * * * * * * * * * PAGE FLIPPING * * * * * * * * * * * * * * *
 * flip () [void macro]                                                  *
 * split (y) [void] [disabled]                                           *
 *   splits the screen into two horizontal parts, the upper of size y/2  *
 * disp (pos) [void macro] [disabled]                                    *
 *   makes the part of the screen starting at pos visible                *
 *                                                                       *
 * * * * * * * * * * * * * * * *  SPRITES  * * * * * * * * * * * * * * * *
 * sputasm (x,y,array) [void asm disabled]                               *
 *   does the same as sput, only in assembly; not noticeably faster      *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


// ========================== INITIALIZATIONS ==========================
#define MAXX 320
#define MAXY 200
unsigned int gpage = 3207;
char _findgfx () {
  char n;
  asm { mov ax, 0x1a00; int 0x10; mov n, al }
  return (n == 0x1a);
}
void _gfxmode () {
  asm {
    mov ax, 0x13; int 0x10
    mov dx, 0x3c4; mov al, 4; out dx, al
    inc dx; in al, dx; and al, 0xf7; or al, 0x04; out dx, al
    mov dx, 0x3d4; mov al, 0x14; out dx, al
    inc dx; in al, dx; and al, 0xbf; out dx, al
    mov dx, 0x3d4; mov al, 0x17; out dx, al
    inc dx; in al, dx; or al, 0x40; out dx, al
    mov dx, 0x3d4; mov al, 0x13; out dx, al
    inc dx; mov ax, 0x40; out dx, al
  }
}
#define gfxmode _gfxmode (); cls (0); flip; cls (0);

// ============================== DRAWING ==============================
#define pset(x,y,col) asm { \
  mov ax, y; shl ax, 7; mov bx, ax; mov ax, gpage; add bx, ax;\
  mov ax, x; mov cx, ax; shr ax, 2; add bx, ax;\
  and cx, 0x03; mov ah, 0x01; shl ah, cl;\
  mov dx, 0x3c4; mov al, 2; out dx, ax;\
  mov ax, 0xa000; mov es, ax; mov al, col; mov es:[bx], al}

// not implemented as yet !
#define xpset(x,y,col) asm { \
  mov ax, y; shl ax, 7; mov bx, ax; mov ax, gpage; add bx, ax;\
  mov ax, x; mov cx, ax; shr ax, 2; add bx, ax;\
  and cx, 0x03; mov ah, 0x01; shl ah, cl;\
  mov dx, 0x3c4; mov al, 2; out dx, ax;\
  mov ax, 0xa000; mov es, ax; mov al, col; mov es:[bx], al}
#define dpset(x,y,col) asm { \
  mov ax, y; shl ax, 7; mov bx, ax; mov ax, gpage; xor ax, 0x8500;\
  add bx, ax; mov ax, x; mov cx, ax; shr ax, 2; add bx, ax;\
  and cx, 0x03; mov ah, 0x01; shl ah, cl;\
  mov dx, 0x3c4; mov al, 2; out dx, ax;\
  mov ax, 0xa000; mov es, ax; mov al, c; mov es:[bx], al}
unsigned char point (int x, int y) {
  unsigned char c;
  asm {
    mov ax, y; shl ax, 7; mov bx, ax; mov ax, gpage; add bx, ax;
    mov ax, x; mov cx, ax; shr cx, 2; add bx, cx;
    mov dx, 0x3ce; and ax, 0x03; mov ah, al; mov al, 4; out dx, ax;
    mov ax, 0xa000; mov es, ax; mov al, es:[bx]; mov c, al;
  }
  return c;
}
void cls (char color) {
  asm {
    mov ax, 0xa000; mov es, ax
    mov ax, gpage; mov bx, ax
    mov dx, 0x3c4; mov ax, 0x0f02; out dx, ax
    mov al, color; mov cx, 80; mov dx, 200
    } c1: asm {
    mov es:[bx], al; inc bx
    dec cx; cmp cx, 0; jne c1;
    mov cx, 80; add bx, 48
    dec dx; cmp dx, 0; jne c1;
  }
}
#define pset4(x,y,col) asm { \
  mov ax, y; shl ax, 7; mov bx, ax; mov ax, gpage; add bx, ax;\
  mov ax, x; mov cx, ax; shr ax, 2; add bx, ax;\
  mov dx, 0x3c4; mov ax, 0x020f; out dx, ax;\
  mov ax, 0xa000; mov es, ax; mov al, col; mov es:[bx], al}


// =========================== PAGE FLIPPING ===========================
#define flip vwait; asm {\
  mov bx, gpage; mov dx, 0x03d4; mov al, 0x0c; out dx, al;\
  inc dx; mov al, bh; out dx, al;\
  mov dx, 0x03d4; mov al, 0x0d; out dx, al;\
  inc dx; mov al, bl; out dx, al} gpage ^= 0x8500
/*void split (int size) {
  int n;
  outp (0x3d4, 24); outp (0x3d5, size); size >>= 8;
  outp (0x3d4, 7); n = inp (0x3d5);
  outp (0x3d5, (n & -17) + ((size & 1) << 4));
  outp (0x3d4, 9); n = inp (0x3d5);
  outp (0x3d5, (n & -65) + ((size & 2) << 6));
}
#define disp(pos) vwait; asm {\
  mov bx, pos; mov dx, 0x03d4; mov al, 0x0c; out dx, al;\
  inc dx; mov al, bh; out dx, al;\
  mov dx, 0x03d4; mov al, 0x0d; out dx, al;\
  inc dx; mov al, bl; out dx, al}
*/


// ============================= SPRITES ===============================
/*void sputasm (int x, int y, char *sprite) {
  asm {
    mov ax, 0xa000; mov es, ax
    mov ax, y; shl ax, 7; mov bx, ax; mov ax, gpage; add bx, ax
    mov ax, x; mov cx, ax; shr ax, 2; add bx, ax
    and cx, 0x03; mov ah, 0x01; shl ah, cl; xor al, al; push ax
    mov dx, 0x3c4; mov al, 2; out dx, ax; mov dx, bx
    mov ax, sprite; mov cx, ax; mov ah, 4
    } s1: asm {
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx;
    add dx, 125; dec ah; cmp ah, 0; jne s1;
    pop ax; shl ah, 1; cmp ah, 16; jne s2
    mov ah, 1; inc bx
    } s2: asm {
    push ax; mov dx, 0x3c4; mov al, 2; out dx, ax;
    mov dx, bx; add dx, -387
    mov ah, 4
    } s1b: asm {
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx;
    add dx, 125; dec ah; cmp ah, 0; jne s1b;
    pop ax; shl ah, 1; cmp ah, 16; jne s2b
    mov ah, 1; inc bx
    } s2b: asm {
    push ax; mov dx, 0x3c4; mov al, 2; out dx, ax;
    mov dx, bx; add dx, -387
    mov ah, 4
    } s1c: asm {
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx;
    add dx, 125; dec ah; cmp ah, 0; jne s1c;
    pop ax; shl ah, 1; cmp ah, 16; jne s2c
    mov ah, 1; inc bx
    } s2c: asm {
    mov dx, 0x3c4; mov al, 2; out dx, ax;
    mov dx, bx; add dx, -387
    mov ah, 4;
    } s1d: asm {
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx; inc dx;
    mov bx, cx; mov al,[bx]; mov bx, dx; mov es:[bx], al; inc cx;
    add dx, 125; dec ah; cmp ah, 0; jne s1d;
  }
} */

#include <pjsgfx.h>

// ============================= THE END ===============================
